@@ -10,20 +10,15 @@ def inspect_evaluate(
     answer: Any,
 ) -> EvaluationResult:
     """Evaluate using Inspect-ai's evaluation models.
-    
+
     Args:
         response: The response to evaluate
         answer: The reference answer to compare against
         model_name: The Inspect model to use
         prompt: Optional custom prompt for evaluation
         metrics: Optional list of metrics to evaluate against
-        
+
     Returns:
         EvaluationResult with the evaluation results
     """
-    return EvaluationResult(
-        score=0.0,
-        reason="Inspect evaluation not implemented",
-        mode="inspect"
-    )
-
+    return EvaluationResult(score=0.0, reason="Inspect evaluation not implemented", mode="inspect")
